@@ -1,4 +1,4 @@
-__version__ = '1.1.1'
+__version__ = '1.1.4'
 
 import calendar
 import datetime
@@ -11,6 +11,11 @@ from typing import Union, Optional
 import requests
 from requests.cookies import cookiejar_from_dict
 from requests.models import Response
+from selenium import webdriver
+from selenium.common.exceptions import TimeoutException
+from selenium.webdriver.common.by import By
+from selenium.webdriver.support.expected_conditions import visibility_of_element_located
+from selenium.webdriver.support.ui import WebDriverWait
 
 
 def add_months(date: datetime.date, months: int) -> datetime.date:
@@ -26,6 +31,9 @@ def response_data(response: Response):
     try:
         return response.json()
     except json.decoder.JSONDecodeError:
+        if not response.text:
+            return None
+
         return {'text': response.text}
 
 
@@ -109,6 +117,7 @@ class PythonAnywhereApi:
         return Response(
             status_code=response.status_code,
             error=response.status_code != 204,
+            data=response_data(response)
         )
 
     def list_consoles(self) -> Response:
@@ -202,6 +211,7 @@ class PythonAnywhereApi:
         return Response(
             status_code=response.status_code,
             error=response.status_code not in (200, 201),
+            data=response_data(response)
         )
 
     def delete_file(self, path) -> Response:
@@ -219,6 +229,7 @@ class PythonAnywhereApi:
         return Response(
             status_code=response.status_code,
             error=response.status_code != 204,
+            data=response_data(response)
         )
 
     def can_create_tasks(self) -> Response:
@@ -235,7 +246,7 @@ class PythonAnywhereApi:
 
         return Response(
             status_code=response.status_code,
-            error=False,
+            error=response.status_code != 200,
             data=response_data(response)
         )
 
@@ -278,15 +289,9 @@ class PythonAnywhereApi:
                 data={'message': traceback.format_exc()}
             )
 
-        if response.status_code != 201:
-            return Response(
-                status_code=response.status_code,
-                error=True
-            )
-
         return Response(
             status_code=response.status_code,
-            error=False,
+            error=response.status_code != 201,
             data=response_data(response)
         )
 
@@ -304,7 +309,225 @@ class PythonAnywhereApi:
 
         return Response(
             status_code=response.status_code,
-            error=False,
+            error=response.status_code != 200,
+            data=response_data(response)
+        )
+
+    def reload_app(self, app_name: str):
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/reload/')
+
+        try:
+            response = self.session.post(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        data = response_data(response)
+        error = response.status_code != 200 or not (hasattr(data, 'status') or data.get('status') == 'OK')
+
+        return Response(
+            status_code=response.status_code,
+            error=error,
+            data=data
+        )
+
+    def enable_app(self, app_name: str):
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/enable/')
+
+        try:
+            response = self.session.post(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        data = response_data(response)
+        error = response.status_code != 200 or not (hasattr(data, 'status') or data.get('status') == 'OK')
+
+        return Response(
+            status_code=response.status_code,
+            error=error,
+            data=data
+        )
+
+    def disable_app(self, app_name: str):
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/disable/')
+
+        try:
+            response = self.session.post(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        data = response_data(response)
+        error = response.status_code != 200 or not (hasattr(data, 'status') or data.get('status') == 'OK')
+
+        return Response(
+            status_code=response.status_code,
+            error=error,
+            data=data
+        )
+
+    def get_static_headers(self, app_name: str):
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_headers/')
+
+        try:
+            response = self.session.get(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 200,
+            data=response_data(response)
+        )
+
+    def create_static_header(self, app_name: str, header_url: str, name: str, value: str) -> Response:
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_headers/')
+
+        data = {
+            'url': header_url,
+            'name': name,
+            'value': value,
+        }
+
+        try:
+            response = self.session.post(url, data=data)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 201,
+            data=response_data(response)
+        )
+
+    def delete_static_header(self, app_name: str, header_id: int) -> Response:
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_headers/{header_id}/')
+
+        try:
+            response = self.session.delete(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 204,
+            data=response_data(response)
+        )
+
+    def get_static_header(self, app_name: str, header_id: int) -> Response:
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_headers/{header_id}/')
+
+        try:
+            response = self.session.get(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 200,
+            data=response_data(response)
+        )
+
+    def get_static_paths(self, app_name: str):
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_files/')
+
+        try:
+            response = self.session.get(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 200,
+            data=response_data(response)
+        )
+
+    def create_static_path(self, app_name: str, static_path_url: str, path: str) -> Response:
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_files/')
+
+        data = {
+            'url': static_path_url,
+            'path': path,
+        }
+
+        try:
+            response = self.session.post(url, data=data)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 201,
+            data=response_data(response)
+        )
+
+    def delete_static_path(self, app_name: str, static_path_id: int) -> Response:
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_files/{static_path_id}/')
+
+        try:
+            response = self.session.delete(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 204,
+            data=response_data(response)
+        )
+
+    def get_static_path(self, app_name: str, static_path_id: int) -> Response:
+        url = self.create_url(f'/webapps/{app_name}.pythonanywhere.com/static_files/{static_path_id}/')
+
+        try:
+            response = self.session.get(url)
+        except requests.exceptions.RequestException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+
+        return Response(
+            status_code=response.status_code,
+            error=response.status_code != 200,
             data=response_data(response)
         )
 
@@ -317,6 +540,7 @@ class PythonAnywhereWeb:
         self.password = password
 
         self.session = None
+        self.selenium = None
 
     @staticmethod
     def create_url(uri: str) -> str:
@@ -341,6 +565,18 @@ class PythonAnywhereWeb:
     def load_cookies(self, cookies: dict):
         self.session.cookies.update(cookiejar_from_dict(cookies))
 
+    def setup_selenium(self):
+        options = webdriver.ChromeOptions()
+        options.add_argument("--no-sandbox")
+        options.add_argument("--headless")
+        options.add_argument("--disable-gpu")
+
+        self.selenium = webdriver.Chrome(options=options)
+
+    def add_selenium_cookies(self):
+        for name, value in self.get_cookies().items():
+            self.selenium.add_cookie({'name': name, 'value': value})
+
     def logout(self) -> Response:
         url = self.create_url('/logout/')
 
@@ -594,3 +830,37 @@ class PythonAnywhereWeb:
             status_code=response.status_code,
             error=False
         )
+
+    def start_console(self, console_id: int, timeout: int = 60):
+        url = self.create_url(f'/user/{self.username}/consoles/{console_id}/frame/')
+
+        self.setup_selenium()
+
+        try:
+            self.selenium.get(self.BASE_URL)
+            self.add_selenium_cookies()
+
+            self.selenium.get(url)
+
+            self.selenium.switch_to.frame(0)
+
+            WebDriverWait(self.selenium, timeout).until(
+                visibility_of_element_located((By.XPATH, "//x-row[contains(text(), '~')]"))
+            )
+
+            return Response(error=False)
+
+        except TimeoutException:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': 'Timed out waiting for console to start.'}
+            )
+        except Exception:
+            return Response(
+                status_code=None,
+                error=True,
+                data={'message': traceback.format_exc()}
+            )
+        finally:
+            self.selenium.quit()
